require("sound.lua")

function love.run()
  _r = math.random
  _f = math.floor
  _ce = math.ceil
  _c = math.cos
  _s = math.sin
  _sq = math.sqrt
  _at2 = math.atan2
  _d2r = math.rad
  _r2d = math.deg
  _m = math.mod
  pi = math.pi
  gr = love.graphics
  ti = love.timer
  ms = love.mouse
  kb = love.keyboard
  math.randomseed(os.time())

	love.load(arg)
	while true do
	  dt = ti.getDelta()
	  time = ti.getTime()
	  fps = love.timer.getFPS()
		ti.step()
		gr.clear()
		love.update(dt)
		love.draw(dt)
		if love.event then
			for e,a,b,c in love.event.poll() do
				if e == "q" then
					if love.audio then love.audio.stop() end
				return
				end
				love.handlers[e](a,b,c)
			end
		end
		ti.sleep(1)
		gr.present()
	end
end

function love.load()
	debugVar = false
  gr.setLineWidth(1)

	imgtiles = gr.newImage("tiles.png")
	imgplayer = gr.newImage("player.png")
	imgenemies = gr.newImage("enemies.png")
	imgcoin = gr.newImage("coin.png")

	sfx = {}
	sfx["coin"] = love.sound.newSoundData("Coin.wav")
	sfx["jump"] = love.sound.newSoundData("Jump.wav")
	sfx["block"] = love.sound.newSoundData("Block.wav")
	sfx["stomp"] = love.sound.newSoundData("Stomp.wav")
	sfx["bump"] = love.sound.newSoundData("Bump.wav")

	tileGrid = {}
	for x=0,imgtiles:getWidth()/32-1 do
		tileGrid[x] = gr.newQuad(x * 32, 0, 32, 32, imgtiles:getWidth(), imgtiles:getHeight())
	end

	tileGrid[100] = gr.newQuad(0, 32, 32, 32, imgtiles:getWidth(), imgtiles:getHeight())
	tileGrid[101] = gr.newQuad(32, 32, 32, 32, imgtiles:getWidth(), imgtiles:getHeight())

	playerGrid = {}
	for x=0,6 do
	  playerGrid[x] = {}
  	for y=0,2 do
  		playerGrid[x][y] = gr.newQuad(x * 32, (64*y), 32, 64, imgplayer:getWidth(), imgplayer:getHeight())
	  end
	end
	playerHead = gr.newQuad(4, 32, 24, 14, imgplayer:getWidth(), imgplayer:getHeight())

	enemyGrid = {}
	enemyGrid["goomba"] = {}
	for x=0,6 do
    enemyGrid["goomba"][x] = gr.newQuad(x * 32, 0, 32, 64, imgenemies:getWidth(), imgenemies:getHeight())
	end


	coinGrid = {}
	for y=0,5 do
		coinGrid[y] = gr.newQuad(0, y * 32, 32, 32, imgcoin:getWidth(), imgcoin:getHeight())
	end

	decoGrid = {}
	decoGrid[1] = gr.newQuad(32, 32, 32, 64, imgtiles:getWidth(), imgtiles:getHeight())
	decoGrid[2] = gr.newQuad(64, 32, 64, 96, imgtiles:getWidth(), imgtiles:getHeight())
	decoGrid[3] = gr.newQuad(32, 96, 32, 32, imgtiles:getWidth(), imgtiles:getHeight())
	decoGrid[4] = gr.newQuad(128, 32, 96, 16, imgtiles:getWidth(), imgtiles:getHeight())

	level = {}

	brbr = {}
	coin = {}
	scor = {}

	hitBlock = function(x,y)
		local t = level.tile[x][y]
		if t.kind == 3 then
			if object["player"].state > 0 then
			  t.hit = 0
        brbr[#brbr+1] = {xd = -128, yd = -256, x = x*32+8, y = y*32+8}
        brbr[#brbr+1] = {xd = 128, yd = -256, x = x*32+24, y = y*32+8}
        brbr[#brbr+1] = {xd = -128, yd = -128, x = x*32+8, y = y*32+24}
        brbr[#brbr+1] = {xd = 128, yd = -128, x = x*32+24, y = y*32+24}
        t.go = nil
  			object["player"].score = object["player"].score + 50
        playSound("block")
      else
        playSound("bump")
			end
			if level.tile[x][y-1].coin then
			  playSound("coin")
				level.tile[x][y-1].coin = false
				coin[#coin+1] = {x = x * 32 + 16, y = (y-2) * 32, life = .26, ys = -20 }
			end
		elseif t.kind == 2 then
			if t.hittimer == 0 then t.hittimer = time + 3 end
			if time >= t.hittimer then
				t.go = nil
				t.hit = 4
			end
      playSound("coin")
			coin[#coin+1] = {x = x * 32 + 16, y = (y-1) * 32, life = .26, ys = -20 }
			object["player"].score = object["player"].score + 200
			object["player"].coins = object["player"].coins + 1
			if level.tile[x][y-1].coin then
				level.tile[x][y-1].coin = false
				coin[#coin+1] = {x = x * 32 + 16, y = (y-2) * 32, life = .26, ys = -20 }
			end
		elseif t.kind == 1 then
			t.hit = 4
			t.go = nil
      playSound("coin")
			coin[#coin+1] = {x = x * 32 + 16, y = (y-1) * 32, life = .26, ys = -20 }
			object["player"].score = object["player"].score + 200
			object["player"].coins = object["player"].coins + 1
			if level.tile[x][y-1].coin then
				level.tile[x][y-1].coin = false
				coin[#coin+1] = {x = x * 32 + 16, y = (y-2) * 32, life = .26, ys = -20 }
			end
		end
	end

	fsmall = gr.newFont("status.ttf", 13)
	fbig = gr.newFont("status.ttf", 40)
	fontStatus = gr.newFont("status.ttf", 28)

	level.time = 300
	level.coinspin = 0
	level.lbounds = 0
	level.rbounds = 0
	level.offset = 0
	level.width = 60

	level.tile = {}
	for x=0,level.width do
		level.tile[x] = {}
		for y=-10,18 do
			level.tile[x][y] = { hit = 0 }
		end
	end
	for y=-10,15 do
		level.tile[level.width][y].hit = 2
	end
	for y=-10,15 do
		level.tile[0][y].hit = 2
	end
	for x=0,level.width do
		level.tile[x][14].hit = 1
		level.tile[x][15].hit = 8
	end
	for x=10,18 do
		level.tile[x][13].coin = true
	end
	for x=1,8 do
		level.tile[x][13].hit = 2
	end
	for x=1,4 do
		level.tile[x][12].hit = 2
	end

	local h
	for x=10,12 do
		for y=8,10 do
			if y == 8 then h = 6 else h = 5 end
			level.tile[x][y] = { hit = h, kind = 3, go = hitBlock }
		end
	end
	level.tile[11][8] = { hit = 6, go = hitBlock, hittimer = 0, kind = 2 }

	level.tile[10][7].coin = true
	level.tile[11][7].coin = true
	level.tile[12][7].coin = true

	for y=14,15 do
		level.tile[26][y].hit = 0
	end
	for y=14,15 do
		level.tile[25][y].hit = 0
	end
	for y=14,15 do
		level.tile[21][y].hit = 0
	end
	for y=14,15 do
		level.tile[23][y].hit = 0
	end

  for x=32,44 do
    for y=14,15 do
      level.tile[x][y].hit = 0
    end
  end

	for x=27,31 do
		level.tile[x][13].hit = 7
	end
	for x=28,30 do
		level.tile[x][12].hit = 7
	end
	level.tile[29][11].hit = 7

	for x=8,9 do
		level.tile[x][6] = { hit = 2 }
	end
	for x=4,6 do
		level.tile[x][4] = { hit = 2 }
	end
	level.tile[4][2] = { hit = 2 }


	level.tile[16][8] = { hit = 2 }

	level.tile[18][10] = { hit = 3, go = hitBlock, hittimer = 0, kind = 2 }
	level.tile[19][10] = { hit = 3, go = hitBlock, hittimer = 0, kind = 1 }

	level.tile[1][11].deco = 3
	level.tile[2][11].deco = 3
	level.tile[13][12].deco = 1
	level.tile[15][11].deco = 2

	for x=0,level.width do
		for y=16,18 do
			level.tile[x][y].hit = level.tile[x][14].hit
		end
	end
	for x=0,level.width do
		for y=-10,-1 do
			level.tile[x][y].hit = level.tile[x][0].hit
		end
	end

	level.platform = {}

	level.platform[#level.platform+1] = { x = 23*32, y = 6*32, w = 96, h = 16, ub=100, lb=300, type = "vert", dir = 1, wait = 0, waittime = .5 }
	level.platform[#level.platform+1] = { x = 20*32, y = 4*32, w = 96, h = 16, ub=14*32, lb=18*32, type = "horiz", dir = -1, wait = 0, waittime = .5 }

	level.platform[#level.platform+1] = { x = 40*32, y = -1*32, w = 96, h = 16, ub=-1*32, lb=17*32, type = "elevvert", dir = -1, wait = 0, waittime = 0 }
	level.platform[#level.platform+1] = { x = 40*32, y = 5*32, w = 96, h = 16, ub=-1*32, lb=17*32, type = "elevvert", dir = -1, wait = 0, waittime = 0 }
	level.platform[#level.platform+1] = { x = 40*32, y = 11*32, w = 96, h = 16, ub=-1*32, lb=17*32, type = "elevvert", dir = -1, wait = 0, waittime = 0 }

	level.platform[#level.platform+1] = { x = 34*32, y = -1*32, w = 96, h = 16, ub=-1*32, lb=17*32, type = "elevvert", dir = 1, wait = 0, waittime = 0 }
	level.platform[#level.platform+1] = { x = 34*32, y = 5*32, w = 96, h = 16, ub=-1*32, lb=17*32, type = "elevvert", dir = 1, wait = 0, waittime = 0 }
	level.platform[#level.platform+1] = { x = 34*32, y = 11*32, w = 96, h = 16, ub=-1*32, lb=17*32, type = "elevvert", dir = 1, wait = 0, waittime = 0 }

  object = {}
	object["player"] = {
	  type = "player",
	  id = "player",
		x = 32 * 3.5,
		y = 11 * 32,
		facing = -1,
		walkframe = 1,
		walkframetotal = 3,
		score = 0,
		coins = 0,
		lives = 5,
		cx = nil,
		cy = nil,
		xs = 0,
		ys = 0,
		bh = 1,
		bw = 1,
		bd = nil,
		bu = nil,
		bl = nil,
		br = nil,
		hitx = nil,
		hity = nil,
		onfloor = false,
		jumping = false,
		jumppow = 0,
		onwall = 0,
		jumpmaxheight = 0,
		jumpheight = 0,
		jumpspeed = 0,
		falling = false,
		fallspeed = 0,
		fallmaxspeed = 512,
		onplatform = 0,
    state = 0,
    jumpspeed = 0,
    jumpmaxspeed = 512,
    jumpstart = 0,
    ismoving = 0,
    isducking = false,
    hvel = 0,
    hmaxvel = 64,
    turning = 0,
    jumppressed = false,
    jj = 0
	}

	object["enemy_1"] = {
	  type = "enemy",
	  pattern = "oneway",
	  appearance = "goomba",
	  id = "enemy_1",
		x = 5 * 32,
		y = 2 * 32,
		facing = -1,
		walkframe = 1,
		walkframetotal = 2,
		score = 0,
		coins = 0,
		lives = 5,
		cx = nil,
		cy = nil,
		xs = 0,
		ys = 0,
		bh = 1,
		bw = 1,
		bd = nil,
		bu = nil,
		bl = nil,
		br = nil,
		hitx = nil,
		hity = nil,
		onfloor = false,
		jumping = false,
		jumppow = 0,
		onwall = 0,
		jumpmaxheight = 0,
		jumpheight = 0,
		jumpspeed = 0,
		falling = false,
		fallspeed = 0,
		fallmaxspeed = 512,
		onplatform = 0,
    state = 0,
    jumpspeed = 0,
    jumpmaxspeed = 512,
    jumpstart = 0,
    ismoving = 0,
    isducking = false,
    hvel = 0,
    hmaxvel = 4,
    turning = 0,
    jumppressed = false,
    jj = 0
	}

	object["enemy_2"] = {
	  type = "enemy",
	  pattern = "oneway",
	  appearance = "goomba",
	  id = "enemy_2",
		x = 10 * 32,
		y = 2 * 32,
		facing = -1,
		walkframe = 1,
		walkframetotal = 2,
		score = 0,
		coins = 0,
		lives = 5,
		cx = nil,
		cy = nil,
		xs = 0,
		ys = 0,
		bh = 1,
		bw = 1,
		bd = nil,
		bu = nil,
		bl = nil,
		br = nil,
		hitx = nil,
		hity = nil,
		onfloor = false,
		jumping = false,
		jumppow = 0,
		onwall = 0,
		jumpmaxheight = 0,
		jumpheight = 0,
		jumpspeed = 0,
		falling = false,
		fallspeed = 0,
		fallmaxspeed = 512,
		onplatform = 0,
    state = 0,
    jumpspeed = 0,
    jumpmaxspeed = 512,
    jumpstart = 0,
    ismoving = 0,
    isducking = false,
    hvel = 0,
    hmaxvel = 4,
    turning = 0,
    jumppressed = false,
    jj = 0
	}


	gr.setBackgroundColor(0,44,116)
	--gr.setBackgroundColor(109,207,246)

	k = {}
	k["left"] = false
end

function love.update(dt)
	for i,p in pairs(level.platform) do
		p.wait = p.wait - dt
		if p.wait < 0 then
			p.wait = 0
		end
		if p.wait == 0 then
			if p.type == "vert" then
				p.y = p.y + (p.dir * 64 * dt)
			elseif p.type == "horiz" then
				p.x = p.x + (p.dir * 64 * dt)
			elseif p.type == "elevvert" then
				p.y = p.y + (p.dir * 64 * dt)
			end
		end
		if p.type == "vert" then
			if p.y > p.lb then
				p.y = p.lb
				p.wait = p.waittime
				if p.dir == 1 then p.dir = -1 else p.dir = 1 end
			end
			if p.y < p.ub then
				p.y = p.ub
				p.wait = p.waittime
				if p.dir == 1 then p.dir = -1 else p.dir = 1 end
			end
		elseif p.type == "horiz" then
			if p.x > p.lb then
				p.x = p.lb
				p.wait = p.waittime
				if p.dir == 1 then p.dir = -1 else p.dir = 1 end
			end
			if p.x < p.ub then
				p.x = p.ub
				p.wait = p.waittime
				if p.dir == 1 then p.dir = -1 else p.dir = 1 end
			end
		elseif p.type == "elevvert" then

			if p.y > p.lb then
				p.y = p.ub
			end
			if p.y < p.ub then
				p.y = p.lb
			end
		end
	end

	level.time = level.time - (dt*2)
	level.coinspin = (level.coinspin + 12 * dt)
	if level.coinspin > 5 then level.coinspin = 0 end

	level.offset = _f(object["player"].x+16 - 400)
	if level.offset < 0 then level.offset = 0 end
	if level.offset > (level.width*32)-(24*32) then level.offset = (level.width*32)-(24*32) end

	level.lbounds = math.max(_f(level.offset / 32)-1,0)
	level.rbounds = math.min(level.lbounds + 26,level.width)

  for oo, obj in pairs(object) do
    if obj.type == "player" then
      kLeft = kb.isDown("left")
      kRight = kb.isDown("right")
      kUp = kb.isDown("up")
      kDown = kb.isDown("down")
      kJump = kb.isDown(" ")
      kShift = kb.isDown("lshift")
    elseif obj.type == "enemy" then
      kLeft = false
      kRight = true
      kUp = false
      kDown = false
      kJump = false
      kShift = false
    end

    obj.cx = obj.x
    obj.cy = obj.y

    local suphei = 0
    f = false
    obj.br = level.width * 32
    for i=_f(obj.cx),level.width * 32,32 do
      if f == false then
        if obj.state > 0 and obj.isducking == false then suphei = level.tile[math.min(_f(i/32),level.width)][_f(obj.y/32)-1].hit end
        if level.tile[math.min(_f(i/32),level.width)][_f(obj.y/32)].hit > 0 or level.tile[math.min(_f(i/32),level.width)][_f((obj.y+24)/32)].hit > 0 or suphei > 0 then
          obj.br = _f(i/32) * 32
          f = true
        end
      end
    end

    f = false
    for i=_f(obj.cx),0,-32 do
      if f == false then
        if obj.state > 0 and obj.isducking == false then suphei = level.tile[math.min(_f(i/32),level.width)][_f(obj.y/32)-1].hit end
        if level.tile[math.min(_f(i/32),level.width)][_f(obj.y/32)].hit > 0 or level.tile[math.min(_f(i/32),level.width)][_f((obj.y+24)/32)].hit > 0 or suphei > 0 then
          obj.bl = _f(i/32) * 32 + 32
          f = true
        end
      end
    end

    f = false
    obj.bd = 600
    for i=_f(obj.y),600,32 do
      if f == false then
        if level.tile[_f((obj.cx+1)/32)][math.min(_f(i/32),15)].hit > 0 or level.tile[_f((obj.cx+31)/32)][math.min(_f(i/32),15)].hit > 0 then
          obj.bd = _f(i/32) * 32
          f = true
        end
      end
    end
    for i=_f(obj.y),600,32 do
      for i,p in pairs(level.platform) do
        if obj.cx > p.x-28 and obj.cx < p.x+p.w-4 and obj.y < p.y then
          if _f(p.y) < obj.bd then obj.bd = _f(p.y) end
        end
      end
    end

    suphei = 32
    f = false
    obj.bu = -200
    if obj.y > 0 then
      for i=_f(obj.y),0,-32 do
        if obj.state > 0 and obj.isducking == false then suphei = 64 end
        if f == false then
          if level.tile[_f((obj.cx+4)/32)][math.min(_f(i/32),15)].hit > 0 or level.tile[_f((obj.cx+27)/32)][math.min(_f(i/32),15)].hit > 0 then
            obj.bu = _f(i/32) * 32 + suphei
            f = true
          end
        end
      end
    end

    local f1 = -1
    for i, obj2 in pairs(object) do
      if obj.id == "player" and obj2.id ~= "player" then
        --print(obj.id, obj2.id)
        local col = overlap(obj.x,obj.y+24,32,8,obj2.x+4,obj2.y+24,24,8)
        local stomp = overlap(obj.x+14,obj.y+28,4,4,obj2.x,obj2.y+4,32,12)
        if col then
          print(obj.id, obj2.id, time)
        elseif stomp then

          playSound("stomp")
          print("Stomp "..time)
          kJump = true
          obj.chain = obj.chain + 1
          obj.jumpstart = obj.cy
          obj.jumppressed = true
          obj.jumping = true
          obj.jumpspeed = obj.jumpmaxspeed
          obj.onfloor = false
          obj.falling = false
          obj.onplatform = 0
          obj.jumpdistance = 32
          if obj.ismoving == 0 then
            obj.jj = 1152+32*2
          elseif obj.ismoving == 1 then
            obj.jj = 1088+32*2
          elseif obj.ismoving == 2 then
            obj.jj = 1024+32*2
          end

        end
        --gr.line(obj.x-level.offset,obj.y,obj2.x-level.offset,obj2.y)
      end
    end

    if obj.cy+32 >= obj.bd then
      obj.onfloor = true
      obj.chain = 0
      obj.onplatform = 0
      obj.falling = false
      obj.jumping = false
      if obj.cy+32 > obj.bd then
        obj.cy = obj.bd-32
      end
    else
      obj.onfloor = false
    end
    obj.onplatform = 0
    for pl, p in pairs(level.platform) do
      if obj.x >= p.x-30 and obj.y >= p.y-32-2 and obj.x <= p.x+p.w-2 and obj.y <= p.y+p.h-32 then
        obj.onplatform = pl
      end
    end

    if kDown then
      if obj.onfloor then obj.isducking = true end
    else
      if obj.onfloor then obj.isducking = false end
    end

    if kShift then obj.hmaxvel = 5 else obj.hmaxvel = 3 end

    if ((kLeft and obj.facing == -1) or (kRight and obj.facing == 1)) and obj.ismoving > 0 then
      obj.turning = .15
      obj.hvel = 0
    end
    obj.turning = obj.turning - dt
    if obj.turning < 0 then obj.turning = 0 end

    if obj.isducking == true then
      local hmv = obj.hmaxvel / 2
      if kLeft and not kRight and obj.jumping == true then
        if obj.isducking == false or obj.onfloor == false then
          obj.hvel = obj.hvel + 12 * dt
          if obj.hvel > hmv then obj.hvel = hmv end
          obj.ismoving = 1
          obj.facing = 1
        end
      elseif kRight and not kLeft and obj.jumping == true then
        if obj.isducking == false or obj.onfloor == false then
          obj.hvel = obj.hvel + 8 * dt
          if obj.hvel > hmv then obj.hvel = hmv end
          obj.ismoving = 1
          obj.facing = -1
        end
      elseif obj.jumping == false then
        obj.hvel = obj.hvel - 8 * dt
        if obj.hvel < 0 then obj.hvel = 0
          obj.ismoving = 0
        end
      end
    else
      if kLeft and not kRight then
        if obj.isducking == false or obj.onfloor == false then
          obj.hvel = obj.hvel + 12 * dt
          if obj.hvel > obj.hmaxvel then obj.hvel = obj.hmaxvel end
          obj.ismoving = 1
          obj.facing = 1
        end
      elseif kRight and not kLeft then
        if obj.isducking == false or obj.onfloor == false then
          obj.hvel = obj.hvel + 8 * dt
          if obj.hvel > obj.hmaxvel then obj.hvel = obj.hmaxvel end
          obj.ismoving = 1
          obj.facing = -1
        end
      elseif obj.jumping == false then
        obj.hvel = obj.hvel - 8 * dt
        if obj.hvel < 0 then obj.hvel = 0
          obj.ismoving = 0
        end
      end
    end

    if kJump and (obj.onfloor or obj.onplatform > 0 or obj.onwall) and obj.jumping == false then
      if obj.jumppressed == false then
        playSound("jump")
        print("Jump "..time)
        obj.jumpstart = obj.cy
        obj.jumppressed = true
        obj.jumping = true
        obj.jumpspeed = obj.jumpmaxspeed
        obj.onfloor = false
        obj.chain = 0
        obj.onplatform = 0
        obj.jumpdistance = 32
        if obj.ismoving == 0 then
          obj.jj = 1152+32
        elseif obj.ismoving == 1 then
          obj.jj = 1088+32
        elseif obj.ismoving == 2 then
          obj.jj = 1024+32
        end
      end
    elseif not kJump then
      obj.jumppressed = false
    end

    if kJump then
      obj.jj = obj.jj - 1152 * dt
      if obj.jj < 512 then obj.jj = 512 end
    end


    local ts
    if obj.turning == 0 then
      ts = obj.hvel
    else
      ts = obj.hvel / 2
    end
    if obj.facing == -1 then
      obj.cx = obj.cx + ts
    else
      obj.cx = obj.cx - ts
    end

    if obj.onfloor == false and obj.jumping == false and obj.onplatform == 0 then
      obj.falling = true end

    if obj.onfloor == false and obj.falling == true then
      obj.jj = 1024
      obj.fallspeed = obj.fallspeed + obj.jj * dt
      if obj.fallspeed > obj.fallmaxspeed then obj.fallspeed = obj.fallmaxspeed end
      obj.cy = obj.cy + obj.fallspeed * dt
    end
    if obj.jumping == true and obj.falling == false and obj.onplatform == 0 then
      obj.jumpamount = obj.jumpstart - obj.cy
      obj.jumpspeed = obj.jumpspeed - obj.jj * dt
      if obj.jumpspeed <= 0 then
        obj.jumpspeed = 0
        obj.fallspeed = 0
        obj.falling = true
        print("Height: "..obj.jumpamount)
      end
      obj.cy = obj.cy - obj.jumpspeed * dt
    end


    if obj.ismoving > 0 then
      local fa = 8
      if obj.ismoving == 2 then fa = 16 end
      obj.walkframe = obj.walkframe + fa * dt
      if obj.walkframe >= obj.walkframetotal+1 then obj.walkframe = 1 end
    end

    if obj.cx > obj.br-32 then
      obj.cx = obj.br-32
      obj.onwall = .25
      obj.hvel = obj.hvel * .9
    end

    if obj.cx < obj.bl then
      obj.cx = obj.bl
      obj.onwall = -.25
      obj.hvel = obj.hvel * .9
    end

    if obj.onwall ~= 0 then
      if obj.onwall < 0 then
        obj.onwall = obj.onwall + dt
        if obj.onwall > 0 then obj.onwall = 0 end
      end
      if obj.onwall > 0 then
        obj.onwall = obj.onwall - dt
        if obj.onwall < 0 then obj.onwall = 0 end
      end
    end

    if obj.onplatform > 0 then
      local dy = level.platform[obj.onplatform].dir
      if level.platform[obj.onplatform].wait == 0 then
        if level.platform[obj.onplatform].type == "vert" then
          obj.cy = level.platform[obj.onplatform].y-32
        end
        if level.platform[obj.onplatform].type == "horiz" then
          obj.cx = obj.cx + (level.platform[obj.onplatform].dir * 64 * dt)
        end
        if level.platform[obj.onplatform].type == "elevvert" then
          obj.cy = level.platform[obj.onplatform].y-32
        end
      end
    else
      if obj.isfalling then
        if obj.cy > obj.bd-32 then
          obj.cy = obj.cy - 4 * dt
          if obj.cy < obj.bd-32 then
            obj.cy = obj.bd-32
          end
        end
      end
    end

    obj.x = obj.cx

    local plyexh = math.min(obj.state + 1,2)
    if obj.isducking then
      plyexh = 1
      obj.hvel = math.max(obj.hvel - dt,0)
    end
    if obj.cy < obj.bu then
      obj.cy = obj.bu --Make sure the player doesn't leave the Y upper bound
      obj.jumppow = 0		--Set the jump power (Gravity) to 0 so you start falling immediately
  --		obj.jumping = false
      obj.falling = true
      obj.jumpheight = 0
      obj.jumpmaxheight = 0
      obj.jumpspeed = 0
      obj.fallspeed = 0
      if level.tile[_f((obj.cx+16)/32)][_f(obj.bu/32)-plyexh].hit > 0 then --Determine which block was hit with your head
        obj.hitx = _f((obj.cx+16)/32)
        obj.hity = _f(obj.bu/32)-plyexh
        obj.hitt = .05
      elseif level.tile[_f((obj.cx)/32)][_f(obj.bu/32)-plyexh].hit > 0 then --Determine which block was hit with your head
        obj.hitx = _f((obj.cx)/32)
        obj.hity = _f(obj.bu/32)-plyexh
        obj.hitt = .05
      elseif level.tile[_f((obj.cx+31)/32)][_f(obj.bu/32)-plyexh].hit > 0 then --Determine which block was hit with your head
        obj.hitx = _f((obj.cx+31)/32)
        obj.hity = _f(obj.bu/32)-plyexh
        obj.hitt = .05
      end
      if obj.hitx and obj.hity then
        if level.tile[obj.hitx][obj.hity].go and obj.y < 500 then
          level.tile[obj.hitx][obj.hity].go(obj.hitx,obj.hity)
        end
      end
    end

    if obj.hitt then
      obj.hitt = obj.hitt - dt
      if obj.hitt < 0 then
        obj.hitt = 0
        obj.hitx = nil
        obj.hity = nil
      end
    end

    obj.y = obj.cy

    if obj.type == "player" then
      local x, y = _f((obj.x+16)/32), _f((obj.y+16)/32)
      if level.tile[x][y].coin then
        level.tile[x][y].coin = false
        obj.coins = obj.coins + 1
        obj.score = obj.score + 200
			  playSound("coin")
      end
    end

    if obj.y > 550 then
      obj.x = 32 * 3.5
      obj.y = 11 * 32
      obj.facing = -1
      obj.lives = obj.lives - 1
    end
  end
end

function love.draw()
	local x2 = 4
	local y2 = 4

	for x=level.lbounds,level.rbounds do
		for y=-1,15 do
			xx = _f((x * 32) - level.offset)
			yy = _f(y * 32)
			--if obj.hitt and y == obj.hity and x == obj.hitx then yy = yy - 8 end
			gr.setColor(0,0,0,100)
			if level.tile[x][y].hit > 0 then
				local r = 0
				--if level.tile[x][y].hit == 3 then r = _s(time * 5)/16 end
				gr.drawq(imgtiles, tileGrid[level.tile[x][y].hit], xx+x2+16, yy+y2+16, r, 1, 1, 16, 16)
			end
		end
	end

	for x=level.lbounds,level.rbounds do
		for y=0,15 do
			xx = _f(x * 32 - level.offset)
			yy = _f(y * 32)
			if object["player"].hitt and y == object["player"].hity and x == object["player"].hitx and (level.tile[x][y].hit >= 3 and level.tile[x][y].hit <= 6) then yy = yy - 8 end
			gr.setColor(255,255,255)
			if level.tile[x][y].hit > 0 then
				local r = 0
				--if level.tile[x][y].hit == 3 then r = _s(time * 5)/16 end
				gr.drawq(imgtiles, tileGrid[level.tile[x][y].hit], xx+16, yy+16, r, 1, 1, 16, 16)
			end

			if level.tile[x][y].deco then
				gr.drawq(imgtiles, decoGrid[level.tile[x][y].deco], xx+16, yy, 0, 1, 1, 16, 0)
			end

			if level.tile[x][y].coin then
				local cs = _f(level.coinspin)
				gr.drawq(imgcoin, coinGrid[cs], xx+16, yy, 0, 1, 1, 16, 0)
			end
		end
	end

	for _, b in pairs(brbr) do
		b.yd = b.yd + 1024 * dt
		b.x = _f(b.x + (b.xd * dt))
		b.y = _f(b.y + (b.yd * dt))
		gr.drawq(imgtiles, tileGrid[100], b.x - level.offset, b.y, 0, 1, 1, 16, 16)
		if b.y > 500 then brbr[_] = nil end
	end

	for _, c in pairs(coin) do
--		gr.drawq(imgtiles, tileGrid[101], c.x, c.y, 0, _s(c.life*30), 1, 16, 0)
		gr.drawq(imgcoin, coinGrid[_f(level.coinspin)], _f(c.x - level.offset), _f(c.y), 0, 1, 1, 16, 0)
		c.ys = c.ys + 128*dt
		c.y = c.y + c.ys
		c.life = c.life - dt
		if c.life < 0 then
			scor[#scor+1] = { x = c.x-32, y = c.y, val = 200, life = 1 }
			coin[_] = nil
		end
	end

  gr.setFont(fontStatus)
	for _, s in pairs(scor) do
    gr.setColor(0,0,0,100)
		gr.printf(s.val, s.x+16 - level.offset+2, s.y+2, 32, "center")
    gr.setColor(255,255,255)
		gr.printf(s.val, s.x+16 - level.offset, s.y, 32, "center")
		s.y = s.y - 32 * dt
		s.life = s.life - dt
		if s.life < 0 then scor[_] = nil end
	end

	for i,p in pairs(level.platform) do
		gr.setColor(255,255,255)
    gr.drawq(imgtiles, decoGrid[4], _f(p.x-level.offset)+(p.w/2), _f(p.y), 0, 1, 1, p.w/2, 0)
	end

  for oo, obj in pairs(object) do
    if debugVar then
      gr.setColor(0,0,0,50)
      gr.rectangle("fill", _f(obj.x/32)*32 - level.offset, _f(obj.y/32)*32, 32, 32)
      if (obj.x/32) ~= _f(obj.x/32) then
        gr.rectangle("fill", (_f(obj.x/32)+1)*32 - level.offset, _f(obj.y/32)*32, 32, 32)
      end
      if (obj.y/32) ~= _f(obj.y/32) then
        gr.rectangle("fill", _f(obj.x/32)*32 - level.offset, (_f(obj.y/32)+1)*32, 32, 32)
      end
      if (obj.x/32) ~= _f(obj.x/32) and (obj.y/32) ~= _f(obj.y/32) then
        gr.rectangle("fill", (_f(obj.x/32)+1)*32 - level.offset, (_f(obj.y/32)+1)*32, 32, 32)
      end

      gr.setColor(0,0,0,50)
      gr.rectangle("fill", obj.bl - level.offset, obj.y, obj.br-obj.bl, 32)
      gr.rectangle("fill", obj.x - level.offset, obj.bu, 32, obj.bd-obj.bu)
    end

    gr.setColor(255,255,255)
    if obj.type == "player" then
      pp = 0
      if obj.ismoving ~= 0 then
        pp = _f(obj.walkframe)
      end
      if obj.turning > 0 then pp = 5 end
      if obj.jumping == true then pp = 4 end
      if obj.isducking == true then pp = 6 end
      s = obj.facing
      gr.drawq(imgplayer, playerGrid[pp][obj.state], _f(obj.x)+16 - level.offset, _f(obj.y)-32, 0, s, 1, 16, 0)
    elseif obj.type == "enemy" then
      pp = obj.appearance
      fn = _f(obj.walkframe-1)
      s = obj.facing
      gr.drawq(
        imgenemies,
        enemyGrid[pp][fn],
        _f(obj.x)+16 - level.offset,
        _f(obj.y)-32,
        0, s, 1, 16, 0)
    end
  end

	gr.setColor(0,0,0,150)
	gr.print("SCORE: " .. formatNumber(object["player"].score), 52, 7)
	gr.setColor(255,255,255)
	gr.print("SCORE: " .. formatNumber(object["player"].score), 50, 5)

	gr.setColor(0,0,0,150)
	gr.print("$" .. formatNumber(object["player"].coins), 282, 7)
	gr.setColor(255,255,255)
	gr.print("$" .. formatNumber(object["player"].coins), 280, 5)

	gr.setColor(0,0,0,150)
	gr.drawq(imgplayer, playerHead, 472, 20, 0, -1, 1, 24, 0)
	gr.print("x"..object["player"].lives, 502, 7)
	gr.setColor(255,255,255)
	gr.drawq(imgplayer, playerHead, 470, 18, 0, -1, 1, 24, 0)
	gr.print("x"..object["player"].lives, 500, 5)

	gr.setColor(0,0,0,150)
	gr.print("TIME: " .. formatNumber(level.time), 622, 7)
	gr.setColor(255,255,255)
	gr.print("TIME: " .. formatNumber(level.time), 620, 5)

	if debugVar then
		gr.setFont(fsmall)
		gr.setColor(255,255,255)
		local t = "Player Bounds D:"..tostring(object["player"].bd).." U:"..tostring(object["player"].bu).." L:"..tostring(object["player"].bl).." R:"..tostring(object["player"].br).."\n"
		--t = t.."PY:"..level.platform[1].y.."\n"
		t = t.."JumpHeight:"..object["player"].jumpheight.."\n"
		t = t.."JumpMaxHeight:"..object["player"].jumpmaxheight.."\n"
		t = t.."JumpSpeed:"..object["player"].jumpspeed.."\n"
		t = t.."Player X:"..object["player"].x.." Y:"..object["player"].y.."\n"
		t = t.."Jumping:"..tostring(object["player"].jumping).."\n"
		t = t.."Jump Power:"..object["player"].jumppow.."\n"
		t = t.."Falling:"..tostring(object["player"].falling).."\n"
		t = t.."FallSpeed:"..tostring(object["player"].fallspeed).."\n"
		t = t.."On Floor:"..tostring(object["player"].onfloor).."\n"
		t = t.."On Wall:"..tostring(object["player"].onwall).."\n"
		t = t.."On Platform:"..tostring(object["player"].onplatform).."\n"
		t = t.."Chain:"..tostring(object["player"].chain).."\n"
		t = t.."BRBR:"..#brbr
		gr.print(t, 38, 40)

		gr.setColor(255,255,255)
		gr.print("FPS: " .. fps, 20, 480)
		gr.print("Bounds L:"..level.lbounds.." R:"..level.rbounds.." Offset:"..level.offset.." "..tostring(k["left"]), 500, 480)
	end
end

function love.keypressed(k)
  if k == "d" then
		debugVar = not debugVar
	elseif k == "g" then
	  object["player"].state = object["player"].state + 1
	  if object["player"].state > 2 then object["player"].state = 0 end
  end
end

function formatNumber(number)
   return (string.format("%d", number):reverse():gsub( "(%d%d%d)" , "%1," ):reverse():gsub("^(-?),","%1"))
end

function playSound(s)
  soundmanager:play(sfx[s])
end

--OVERLAP
function overlap(x1,y1,w1,h1,x2,y2,w2,h2, i)
	local tl, bl, tr, br = false, false, false, false
	if (x2 >= x1 and x2 <= (x1 + w1)) and (y2 >= y1 and y2 <= (y1 + h1)) then tl = true end
	if (x2+w2 >= x1 and x2+w2 <= (x1 + w1)) and (y2 >= y1 and y2 <= (y1 + h1)) then tr = true end
	if (x2 >= x1 and x2 <= (x1 + w1)) and (y2+h2 >= y1 and y2+h2 <= (y1 + h1)) then bl = true end
	if (x2+w2 >= x1 and x2+w2 <= (x1 + w1)) and (y2+h2 >= y1 and y2+h2 <= (y1 + h1)) then br = true end

	if (x1 >= x2 and x1 <= (x2 + w2)) and (y1 >= y2 and y1 <= (y2 + h2)) then tl = true end
	if (x1+w1 >= x2 and x1+w1 <= (x2 + w2)) and (y1 >= y2 and y1 <= (y2 + h2)) then tr = true end
	if (x1 >= x2 and x1 <= (x2 + w2)) and (y1+h1 >= y2 and y1+h1 <= (y2 + h2)) then bl = true end
	if (x1+w1 >= x2 and x1+w1 <= (x2 + w2)) and (y1+h1 >= y2 and y1+h1 <= (y2 + h2)) then br = true end

	if debugVar then
		gr.setColor(255,255,255,100)
		gr.line(x1-level.offset, y1, x2-level.offset, y2)
		gr.setColor(255,0,0,100)
		gr.rectangle("fill", x1-level.offset, y1, w1, h1)
		gr.setColor(0,255,255,100)
		gr.rectangle("fill", x2-level.offset, y2, w2, h2)
		gr.setColor(0,0,0,255)
	end
	if tl or tr or bl or br then return true else return false end
end
